#include <MD.h>
#include <time.h>

using namespace ZSET;

void read_buf_list(BUFF_LIST<double>& val, ASCII_FILE& file)
{ val.resize(0);
  for (;;) {
      double dd=file.getdouble();
      if (file.ok) val.add(dd);
      else { file.back(); break; }
  }
}


const double safe_coef_for_cells_x = 1.2; // Must be > 1.
const double safe_coef_for_cells_y = 1.2; // Must be > 1.
const double safe_coef_for_cells_z = 1.2; // Must be > 1.
Zofstream energy_out;
DECLARE_OBJECT(PROBLEM_COMPONENT,MD,molecular_dynamics)
ADD_PB_TYPE(md,molecular_dynamics)


double MD::get_temperature(double time) {
  if (prescribed_temperature.size() > 0) {
    int n = prescribed_temperature.size()/2;
    for (int i = 1; i < n; i++) {
       if (prescribed_temperature[i*2] > time) {
         double t1 =  prescribed_temperature[(i-1)*2],   t2 =  prescribed_temperature[i*2],
	        T1 =  prescribed_temperature[(i-1)*2+1], T2 =  prescribed_temperature[i*2+1];
         return T1 + (T2-T1)*(time-t1)/(t2-t1); 
       }
    }
  if (time < prescribed_temperature[0])
    return prescribed_temperature[1];
  if (time > prescribed_temperature[2*(n-1)])  {
    return prescribed_temperature[2*n-1];
Out << "here" << endl;
  }
 }
 return T0;
}

MD::~MD()
{
}
void MD::read_restart(RST_FSTREAM& /* obin */) { }

void MD::write_restart(RST_FSTREAM& /* obin */) { }

void MD::end_problem()
{
  //energy_out.close();
  //file_displ_force.close();
  output_particles.close();
  output_energy.close();
}
bool MD::initialize()
{
  return TRUE;
}
void MD::load(ASCII_FILE& inp, PROBLEM* boss)
{
  this->its_boss = boss;
  start_problem = TRUE;

  STRING ctl, str;
  STRING potential_name;
  dist_mult = 1.;
 
  when_write_restart = 0;
  set_every=1000000;
  read_from_file = FALSE;  
  prescribed_temperature.resize(0);
  gamma_temperature = 0.5;

  inp.locate_at_level(3,"molecular_dynamics");
  if(!inp.ok) INPUT_ERROR("Cannot find ***molecular_dynamics in file");

  for(ctl=inp.start_key();inp.ok;ctl=inp.next_key()) 
  {
    if (ctl == "num_particles")
    {
      Np = inp.getint();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl == "dim")
    {
      dim = inp.getint();
      if(!inp.ok) INPUT_ERROR("Read failed");
      box.resize(dim);
    }
    else if (ctl == "num_timesteps")
    {
      Ntime = inp.getint();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl == "box_size")
    {
      box = inp.getVECTOR();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl == "distribution")
    {
      distribution = inp.getSTRING();
      if (distribution != "fcc" && distribution != "hcp" && distribution != "bcc")
	ERROR("Unknown lattice type '"+distribution+"', known types are 'fcc', 'hcp', 'bcc'");
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="cut_off") 
    {
      cut_off = inp.getdouble();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="dt") 
    {
      dt = dt0 = inp.getdouble();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="T0") 
    {
      T0 = inp.getdouble();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="temperature_adjustment_coefficient") {
      gamma_temperature = inp.getdouble();
      Out << "gamma read = " << gamma_temperature << endl;
      if(!inp.ok) INPUT_ERROR("Read failed");
      if (gamma_temperature < 0 || gamma_temperature > 1)
	ERROR("temperature adjustment coefficient should be 0 <= c <= 1");
    }
     else if (ctl=="set_temperature") {
      read_buf_list(prescribed_temperature,inp);
      if (prescribed_temperature.size() % 2 != 0)
	ERROR("set_temperature should have an even number of elements...");
    }
    else if (ctl=="adjust_temperature_every") {
      set_every = inp.getint();
    }
    else if (ctl=="eq_distance_mult") 
    {
      dist_mult = inp.getdouble();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="seed") 
    {
      seed = inp.getint();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="integrator") 
    {
      integrator = new MD_INTEGRATOR(inp.getSTRING());
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="configuration") 
    {
      config = inp.getSTRING();
      if (config == "*file")
      {
	read_from_file = TRUE;
        input_file = inp.getSTRING();
      }
      else if (config == "bicrystal") {
	tilt = M_PI*inp.getdouble()/180.;
      }
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="write_restart")
    {
      when_write_restart = inp.getint();
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="md_output") 
    {
      STRING s1,s2;
      int ri = -1;
      double rd = -1.;
      s1 = inp.getSTRING();
      if (s1 == MD_PRINT_TIME) 
      {
	rd = inp.getdouble();
        s2 = inp.getSTRING();
	md_output = new MD_OUTPUT(s1,rd,s2);
      }
      else if (s1 == MD_PRINT_INCR)
      {
	ri = inp.getint();
        s2 = inp.getSTRING();
	md_output = new MD_OUTPUT(s1,ri,s2);
      }
      /*
      md_output = new MD_OUTPUT(inp.getSTRING());
      if (md_output->get_type() == MD_PRINT_TIME)
	 md_output->set_double_time(inp.getdouble());
      else if (md_output->get_type() == MD_PRINT_INCR)
	 md_output->set_int_time(inp.getint());
      md_output->set_print_option(inp.getSTRING());
      */
      if(!inp.ok) INPUT_ERROR("Read failed");
    }
    else if (ctl=="potential")
    {
      potential = new MD_POTENTIAL(inp.getSTRING());
      if (potential->get_name() == LJ)
      {
        str = inp.getSTRING();
        if (str == "*powers")
        {
          potential->set_param(1,inp.getdouble());
          potential->set_param(2,inp.getdouble());
        }
      }
      else if (potential->get_name() == CONTACT)
      {
        str = inp.getSTRING();
        if (str == "*disp_max_force")
	  potential->set_param(1,inp.getdouble());
        else 
	  INPUT_ERROR("Potential <TM> read failed");
        str = inp.getSTRING();
        if (str == "*max_disp")
	  potential->set_param(2,inp.getdouble());
        else 
	  INPUT_ERROR("Potential <TM> read failed");
      }
      else if (potential->get_name() == TM)
      {
        str = inp.getSTRING();
        if (str == "*powers")
        {
          potential->set_param(1,inp.getdouble());
          potential->set_param(2,inp.getdouble());
        }
        else 
	  INPUT_ERROR("Potential <TM> read failed");
        str = inp.getSTRING();
        if (str == "*second_minimum")
        {
          potential->set_param(3,inp.getdouble());
          potential->set_param(4,inp.getdouble());
        }
        else 
	  INPUT_ERROR("Potential <TM> read failed");
      }
    }
    else if (ctl=="MD_material")
    {
      STRING mat_name = inp.getSTRING();
      int mat_id = inp.getint();
      MD_MATERIAL* mat = new MD_MATERIAL(mat_id,mat_name);
      do
      {
	str = inp.getSTRING();
	if (str == "*mass")
	  mat->set_mass(inp.getdouble());
	else if (str == "*color")
	  mat->set_color(inp.getint());
	// LJ-type potential
	else if (str == "*epsilon")
	  mat->set_material_parameter(1,inp.getdouble());
	else if (str == "*sigma")
	  mat->set_material_parameter(2,inp.getdouble());
	// CONTACT Potential
	else if (str == "*radius")
	  mat->set_material_parameter(1,inp.getdouble());
	else if (str == "*Young")
	  mat->set_material_parameter(2,inp.getdouble());
	else if (str == "*Poisson")
	  mat->set_material_parameter(3,inp.getdouble());
      }
      while (str != "**return");
      material.add(mat);
  }
  else if (ctl=="mixing_rule")
  {
       ARRAY<double> mix(4);
       mix[0] = double(inp.getint()); 
       mix[1] = double(inp.getint());
       mix[2] = inp.getdouble();
       mix[3] = inp.getdouble();
       if (potential != NULL && mix[0] <= material.size() && mix[1] <= material.size())
         potential->set_mix_rule(mix);
       else
	 ERROR("Potential should be defined before calling 'mixing_rule' and number of materials should be smaller than "+itoa(int(mix[1]))+" and "+itoa(int(mix[1])));
  }
  else 
    INPUT_ERROR("Unknown keyword: "+ctl);
  }
  
}
void MD::init_problem()
{
  //file_displ_force.open((PRINT_OPTION_FOLDER+STRING("displ_force.md"))(),ios::out);
  if (start_problem)
  {
  pb=(PROBLEM_STATIC_MECHANICAL*)its_boss;
  md_time = 0.;
  md_increment = 0;
  displacement = 0.;
  bc.resize(0);
// Mesh for phase transformation problems 
//  md_mesh = new MD_MESH("fcc"); 


// Reflecting BC
   
  bc.add(new MD_BC(RIGID_X_LEFT));
  bc.add(new MD_BC(RIGID_X_RIGHT));
  bc.add(new MD_BC(RIGID_Y_TOP));
  bc.add(new MD_BC(RIGID_Y_BOT));

// Periodic BC
   /*
  bc.add(new MD_BC(PERIODIC_X));
  bc.add(new MD_BC(PERIODIC_Y));
*/ 
// Print Material
  for (int i = 0; i < material.size(); i++)
    material[i]->print();

// Normalize the material parameters :: scale everything accordingly to the 1st material
  material[0]->set_dim(dim);
  material[0]->set_multiplier();
  material[0]->scale_parameters();
  dt = dt0 /= material[0]->multiplier.time;
  cout << "Set first materials" << endl;
  for (int i = 1; i < material.size(); i++) {
    material[i]->set_dim(dim);
    material[i]->set_multiplier(material[0]->get_multiplier());
    material[i]->scale_parameters();
  }
  cout << "Set materials" << endl;
  if (material.size() > 1)
    potential->scale_parameters(material[0]->get_multiplier());
  md_output->scale(material[0]->get_multiplier().time);
  for (int i = 0; i < prescribed_temperature.size()/2; i++) 
    prescribed_temperature[2*i] /= material[0]->multiplier.time;
  box = box/material[0]->multiplier.length;
  Out << "Box size = " << box << endl;
// File for energy output
  //output_energy.open((PRINT_OPTION_FOLDER+STRING("energy.md"))(),ios::out);
  output_energy.open("energy.md",ios::out);
  output_energy << "# energy scaling = " << material[0]->multiplier.energy << endl; 
  output_energy << "# time scaling = " << material[0]->multiplier.time << endl; 
  output_energy << "# velocity scaling = " << material[0]->multiplier.velocity << endl; 
  output_energy << "# length scaling = " << material[0]->multiplier.length << endl; 
  output_energy << "# time kinetic_energy potential_energy temperature" << endl;
  output_particles.open("frames.md",ios::out);

// tmp
   tmp_output.open("Shear.data",ios::out);
/*
   tmp_output.open("temperature.data",ios::out);
   for (int i = 0; i < 10000; i++)
     tmp_output << material[0]->multiplier.time*dt*i << " " << get_temperature(dt*i) << endl; 
   tmp_output.close();
   abort();
*/ 

// Initialize potentials
  if (material.size() == 1)
    potential->initialize(material[0]);
  else
    potential->initialize();

  set_initial_conf(config); 

// Configure particles and cells
  define_rmin_rmax();
  distribute_into_cells();
  md_output->print(cell);

// Initialize integrator
  integrator->init(cell);
  integrator->update_region_bc(bc);
  if  (potential->get_name() == CONTACT)
    potential->precompute_links(cell, Np);

// Initial distribution of velocities
    if (!read_from_file) {
      Out << "Start MaxwellBoltzman velocity distribution for T = " << get_temperature(0) << endl;
      MaxwellBoltzmanDistribution(get_temperature(0));
      Out << "Finish MaxwellBoltzman velocity distribution" << endl;
    }
    start_problem = FALSE;
  }
}
void MD::start_increment()
{

  for (itime = 0; itime < Ntime; itime++) {
    if (itime % 100 == 0)
       Out << "MD time = " << itime << endl;
    md_start_increment();
    md_end_increment();
  }
}
void MD::md_start_increment()
{
// SOLVE IT!
  update_bc();
  update_dt();
  md_time += dt;
  md_increment++;
  for (int i = 0; i < bc.size(); i++)
    bc[i]->update(md_time);
  integrator->solve_step(cell,potential,dt,md_output->if_print(md_time,itime),kinetic_energy,potential_energy,pre_pressure,Timer_counter);
}
void MD::md_end_increment() {
  Timer_counter.start_time("MD end incr");
  if (md_output->if_print(md_time,itime)) {
    md_output->print(output_particles,cell,md_time,itime);
    //md_output->print(md_mesh,particle);
    //md_output->print_by_cells(cell,Nx);
    //md_output->print_integrals(output_energy,cell,potential,md_time);
    md_output->print_integrals(output_energy, kinetic_energy, potential_energy, pre_pressure, md_time, particle.size(), material[0]->multiplier.energy, dim);
  }
  if (md_increment == when_write_restart)
    md_output->write_restart(particle);
  Timer_counter.start_time("MD end incr");
  
}
void MD::set_initial_conf(STRING str)
{
 if (!read_from_file) {
  VECTOR r(dim),r0(dim),v(dim),dx(dim),dy(dim);

// distribute particles
  if (dim == 2)
  {
    r0[0] = 0.;
    r0[1] = 0.;
    if (potential->get_name() != CONTACT)
      dx[0] = dist_mult*material[0]->material_parameter[1]*pow(2.,1./6.);
    else 
      dx[0] = dist_mult*material[0]->material_parameter[0];
    dx[1] = 0;
    dy[0] = 0;
    if (distribution == "hcp") 
      dy[1] = dx[0]*sqrt(3.)/2.; 
    else if (distribution == "fcc" || distribution == "bcc")
      dy[1] = dx[0];
  }
  else
    ERROR("Not implemented in 3D");
 if (str == "square")
 {
  Npx = int(sqrt(Np)); //150
  Npy = int(Np/double(Npx)); //150
  Np = Npx*Npy + (Npx-1)*(Npy-1);
  particle.resize(Np);
  int sign = 1;
  r = r0;
  int color = 1, material_id;
// White atoms
  for (int j = 0; j < Npy; j++)
  {
    for (int i = 0; i < Npx; i++)
    {
        particle[i+j*Npx] = new PARTICLE(i+j*Npx+1,color,r,v,material[color-1]);
// Mesh for phase transformation elements
/*
        if (i + 1 < Npx && j + 1 < Npy)
        {
          MD_ELEMENT* el = new MD_ELEMENT(md_mesh->size());

	  el->add_particle(i+j*Npx,r,color);
          el->add_particle(i+j*Npx+1,r+dx,color);
          el->add_particle(i+(j+1)*Npx+1,r+dx+dy,color);
          el->add_particle(i+(j+1)*Npx,r+dy,color);

          md_mesh->add_element(el);
        }
*/
        r += dx;
    }
    if (distribution == "hcp")
      r += dy - dx * Npx + sign*dx/2.;
    else if (distribution == "fcc" || distribution == "bcc")
      r += dy - dx * Npx;
    sign*=-1;
  }
// Black atoms
  sign = +1;
  color = 2;
  int PS = Npx*Npy, el_id = 0;
  r = r0 + 0.5*(dx+dy);
  for (int j = 0; j < Npy-1; j++)
  {
    prn("j = ", j);
    for (int i = 0; i < Npx-1; i++)
    {
        prn("i = ", i);
        particle[PS+i+j*(Npx-1)] = new PARTICLE(PS+i+j*(Npx-1)+1,color,r,v,material[color-1]);
// Mesh for phase transformation elements
/*
        if (i + 1 < Npx && j + 1 < Npy)
        {
	   (md_mesh->get_element(el_id))->add_particle(PS+i+j*(Npx-1),r,color);
	   el_id++;
	}
*/
        r += dx;
    }
    if (distribution == "hcp")
      r += dy - dx * Npx + sign*dx/2.;
    else if (distribution == "fcc" || distribution == "bcc")
      r += dy - dx * (Npx-1);
    sign*=-1;
  }
// Mesh for phase transformation elements
/*
  md_mesh->done();
*/

//o tmp
/*
  Zofstream test;
  test.open("tEsT.tmp",ios::out);
  VECTOR rc = md_mesh->get_element(0)->get_coord0(0), rv;
  for (int i = 0; i < md_mesh->size(); i++)
  {
    MD_ELEMENT& el = *(md_mesh->get_element(i));
    rv = el.get_coord0(0);
    if (rv != rc)
      test << endl;
    for (int j = 0; j < el.size()-1; j++)
    {
       VECTOR r = el.get_coord0(j);
       test << r[0] << " " << r[1] << " " << el.get_color(j) << endl;
    }
    test << el.get_coord0(0)[0] << " " << el.get_coord0(0)[1] << endl;
    Out << "rv = " << rv << ", rc = " << rc << endl << endl;
    rc = el.get_coord0(1);
  }
  test.close();
*/
//x tmp
 }
 else if (str == "gas") {
  int id = 1;
  double x,y,c=dx.norm();
  int color = 1, material_id = 0, num_try = 0;
  bool restart;
  VECTOR r(dim);
  if (box.size() == 0) 
    ERROR("To \"**configuration gas\" one needs also to prescribe \"**box (l1,l2)\" containing box dimensions.");
// Insert first particle in the middle
  particle.add( new PARTICLE(id,2,box*0.5,v,material[0]));
// Insert others
  Out << "Num of particles = " << Np << endl;
  do {
      num_try++;
      restart = FALSE;
      x = (0.03+0.94*rand()/double(RAND_MAX))*box[0];
      y = (0.03+0.94*rand()/double(RAND_MAX))*box[1];
      r[0]=x;
      r[1]=y;
      for (int i = 0; i < particle.size(); i++) {
         VECTOR rs = particle[i]->x;
         if ((rs-r).norm() < c) {
	   restart = TRUE;
           break;
	 }
      }
      if (!restart) {
	id++;
        particle.add( new PARTICLE(id,color,r,v,material[material_id]) );
      }
  } while (id < Np  && num_try < 100*Np);
  Out << "num_try = " << num_try << endl; 
  if (num_try == 100*Np) {
    Out << endl << "ERROR: Impossible to insert so many particles ("<< Np << ") in the box " << box << ". I inserted only " << id << " particles..." << endl;
    abort();
  }
 }
 else if (str == "stacking_fault") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx);
  int sign=1;
  Np = Npx*Npy;
  r = r0;
  int id = 0;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        /*if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
*/
	material_id = 0;

	if ( j == int(Npy*0.5) && i > 0.4*Npx && i < 0.6*Npx) { 
          color = 2;
        }
        else {
	  id++;
/*
          if (j == int(Npy*0.5)-1 && i > 0.3*Npx && i < 0.7*Npx) 
            particle.add( new PARTICLE(id,color,r+dy*0.1,v,material[material_id]) );
          else if (j == int(Npy*0.5)+1 && i > 0.3*Npx && i < 0.7*Npx) 
            particle.add( new PARTICLE(id,color,r-dy*0.1,v,material[material_id]) );
          else
*/
            particle.add( new PARTICLE(id,color,r,v,material[material_id]) );
        }
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    sign*=-1;
  }

// Mesh for phase transformation elements
/*
        if (i + 1 < Npx && j + 1 < Npy)
        {
          MD_ELEMENT* el = new MD_ELEMENT(md_mesh->size());

	  el->add_particle(i+j*Npx,r,color);
          el->add_particle(i+j*Npx+1,r+dx,color);
          el->add_particle(i+(j+1)*Npx+1,r+dx+dy,color);
          el->add_particle(i+(j+1)*Npx,r+dy,color);

          md_mesh->add_element(el);
        }
*/
 

 }
/*
 else if (str == "gas") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx/2);
  int sign=1;
  Np = Npx*Npy;
  r = r0;
  int id = 0;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
	material_id = 0;

	  id++;
          particle.add( new PARTICLE(id,color,r,v,material[0]) );
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    sign*=-1;
  }
 }
*/
 else if (str == "drop") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx/2);
  int sign=1;
  Np = Npx*Npy;
  r = r0;
  int id = 0;
  double xc = Npx*dx[0]*0.5, yc = Npy*dx[0]*0.9, y0 = Npy*dx[0]*0.5, R = 0*Npy*dx[0]*0.1;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
	material_id = 0;

	if ( r[1] < y0 || (r[0]-xc)*(r[0]-xc) + (r[1]-yc)*(r[1]-yc) < R*R) {
	  id++;
          particle.add( new PARTICLE(id,color,r,v,material[material_id]) );
          particle.last()->bc_f = -12.*dy/(dy[1]);
        }
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    sign*=-1;
  }
 }

 else if (str == "shear") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx/1.5);
  int sign=1;
  Np = Npx*Npy;
  r = r0;
  int id = 0;
  double xc = Npx*dx[0]*0.5, yc = Npy*dx[0]*sqrt(3)/4., R = Npy*dx[0]*0.1;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
	material_id = 0;

	if (!(i == int(Npx/2.) && j == int(Npy/2.))) { //random()/double(RAND_MAX) > 0.001) {
	  id++;
          particle.add( new PARTICLE(id,color,r,v,material[material_id]) );
        }
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    sign*=-1;
  }
  double length = dx[0]*Npx,yx=0;
  bool added1=FALSE, added2=FALSE;
  for (int i = 0; i < particle.size(); i++) {
    if (particle[i]->x[0]<r0[0]+length*0.1) { 
      particle[i]->color = 3;
      particle[i]->bc_x = particle[i]->x;
      if (!added1 && particle[i]->x[0] > 0.) {
        applied_bc0.add(i);
	added1=TRUE;
        yx = particle[i]->x[1];
      }
    }
    else if (particle[i]->x[0] >r0[0]+length*0.9) { 
      particle[i]->color = 4;
      particle[i]->bc_x = particle[i]->x;
      applied_bc.add(i);
      if ((particle[i]->x[1] - yx) < 1e-6 && !added2) {
        applied_bc0.add(i);
	added2=TRUE;
      }
    }
  }
 }

 else if (str == "hole") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx/1.5);
  int sign=1;
  Np = Npx*Npy;
  r = r0;
  int id = 0;
  double xc = Npx*dx[0]*0.5, yc = Npy*dx[0]*sqrt(3)/4., R = Npy*dx[0]*0.1;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
	material_id = 0;

	if ((r[0]-xc)*(r[0]-xc) + (r[1]-yc)*(r[1]-yc) > R*R && random()/double(RAND_MAX) > 0.01) {
	  id++;
          particle.add( new PARTICLE(id,color,r,v,material[material_id]) );
        }
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    sign*=-1;
  }
  double length = dx[0]*Npx;
  for (int i = 0; i < particle.size(); i++) {
    if (particle[i]->x[0]<r0[0]+length*0.02) { 
      particle[i]->color = 3;
      particle[i]->bc_x = particle[i]->x;
    }
    else if (particle[i]->x[0] >r0[0]+length*0.98) { 
      particle[i]->color = 4;
      particle[i]->bc_x = particle[i]->x;
      applied_bc.add(i);
    }
  }
 }
 else if (str == "rectangle") {
  const double RATIO_NX_TO_NY = 2.;
  Npx = int(sqrt(2.*Np)); 
  Npy = int(Npx/4.);
  int sign=1;
  Np = Npx*Npy;
  particle.resize(Np);
  r = r0;
  int color = 1, material_id;
  for (int j = 0; j < Npy; j++) {
    for (int i = 0; i < Npx; i++) {
        if (color == 1) 
	  color = 2;
        else if (color == 2)
	  color = 1;
	material_id = 0;

        particle[i+j*Npx] = new PARTICLE(i+j*Npx+1,color,r,v,material[material_id]);
        r += sign*dx;
    }
    if (distribution == "hcp")
      r += dy - sign*dx/2.;
    else if (distribution == "fcc" || distribution == "bcc")
      r += dy- sign*dx;
    sign*=-1;
  }
  double length = dx[0]*Npx;
  for (int i = 0; i < particle.size(); i++) {
    if (particle[i]->x[0]<r0[0]+length*0.1) { 
      particle[i]->color = 3;
      particle[i]->bc_x = particle[i]->x;
    }
    else if (particle[i]->x[0] >r0[0]+length*0.9) { 
      particle[i]->color = 4;
      particle[i]->bc_x = particle[i]->x;
      applied_bc.add(i);
    }
  }
 }
 else if (str == "bicrystal") {
  // x=2*y
  // x*y=N
  const double RATIO_NX_TO_NY = 2.;
  Npx = 2000;
  Npy = 2000;
  int sign=1;
  Np = Npx*Npy;
  //particle.presize(Np);
  SMATRIX Rot(2);
  int color = 1, id = 1;
  double min=1e10;
  // crystal 1
  Rot(0,0) = cos(-tilt); Rot(0,1) = -sin(-tilt);
  Rot(1,0) = sin(-tilt); Rot(1,1) = cos(-tilt);
  r = Rot*r0;
  for (int j = 0; j < Npy; j++) {
    color = color == 1 ? 2 : 1;
    for (int i = 0; i < Npx; i++) {
	if (r[0] > 50. && r[0] < 450. && r[1] > -1.e-5 && r[1] < 50.) {
	  if (r[1]<min)
	    min = r[1];
	  particle.add(new PARTICLE(id,color,r,v,material[0]));
	  id++;
        }
        //particle[i+j*Npx] = new PARTICLE(i+j*Npx+1,color,r,v,material[material_id]);
        r += Rot * (sign*dx);
    }
    if (distribution == "hcp")
      r += Rot * (dy - sign*dx/2.);
    else 
      INTERNAL_ERROR;
    sign*=-1;
  }
  // crystal 2
  Out << "Particle.size = " << particle.size() << endl;
  int psize = particle.size();
  for (int ip = 0; ip < psize; ip++) {
    PARTICLE* p = particle[ip];
    if (p->x[1] > 0.5 && (p->x[1]-15.)*(p->x[1]-15.) + (p->x[0]-250.)*(p->x[0]-250.) <= 15.*15.) {
      VECTOR r = p->x;
      r[1] = -r[1]+min-7;
      r[0] =  r[0]-.3;
      if (p->color == 1)  
	color = 3;
      else 
	color = 4;
      particle.add(new PARTICLE(id,color,r,v,material[1]));
      id++;
    }
  }
  Np = id-1;
 }
 else if (str == "bilayer") {
  cout << "here" << endl;
  // x=2*y
  // x*y=N
  const double RATIO_NX_TO_NY = 2.;
  Npx = 2000;
  Npy = 2000;
  int sign=1;
  Np = Npx*Npy;
  //particle.presize(Np);
  SMATRIX Rot(2);
  int color = 1, id = 1;
  double min=1e10;
  // crystal 1
  Rot(0,0) = cos(-tilt); Rot(0,1) = -sin(-tilt);
  Rot(1,0) = sin(-tilt); Rot(1,1) = cos(-tilt);
  r = Rot*r0;
  for (int j = 0; j < Npy; j++) {
    color = color == 1 ? 2 : 1;
    for (int i = 0; i < Npx; i++) {
	if (r[0] > 50. && r[0] < 450. && r[1] > -1.e-5 && r[1] < 50.) {
	  if (r[1]<min)
	    min = r[1];
	  particle.add(new PARTICLE(id,color,r,v,material[0]));
	  id++;
        }
        //particle[i+j*Npx] = new PARTICLE(i+j*Npx+1,color,r,v,material[material_id]);
        r += Rot * (sign*dx);
    }
    if (distribution == "hcp")
      r += Rot * (dy - sign*dx/2.);
    else 
      INTERNAL_ERROR;
    sign*=-1;
  }
  // crystal 2
  Out << "Particle.size = " << particle.size() << endl;
  int psize = particle.size();
  for (int ip = 0; ip < psize; ip++) {
    PARTICLE* p = particle[ip];
    if ( p->x[1] > 50. && p->x[0] > 50. && p->x[0] < 450. && p->x[1] < 100.) { //(p->x[1]-15.)*(p->x[1]-15.) + (p->x[0]-250.)*(p->x[0]-250.) <= 15.*15.) {
      VECTOR r = p->x;
      r[1] = -r[1]+min-7;
      r[0] =  r[0]-.3;
      if (p->color == 1)  
	color = 3;
      else 
	color = 4;
      particle.add(new PARTICLE(id,color,r,v,material[1]));
      id++;
    }
  }
  Np = id-1;
 }
 else if (str == "impact") {
  // x=2*y
  // x*y=N
  const double RATIO_NX_TO_NY = 2.;
  Npx = 20;
  Npy = 20;
  int sign=1;
  Np = Npx*Npy;
  //particle.presize(Np);
  SMATRIX Rot(2);
  int color = 1, id = 1;
  double min=1e10;
  tilt = -M_PI*49./180.;
  // crystal 1
  Rot(0,0) = cos(-tilt); Rot(0,1) = -sin(-tilt);
  Rot(1,0) = sin(-tilt); Rot(1,1) = cos(-tilt);
  r = Rot*r0;
  for (int j = 0; j < Npy; j++) {
    color = color == 1 ? 2 : 1;
    for (int i = 0; i < Npx; i++) {
	if (1) { //r[0] > 100. && r[0] < 120. && r[1] > -1.e-5 && r[1] < 20.) {
	  if (r[1]<min)
	    min = r[1];
	  particle.add(new PARTICLE(id,color,r,v,material[0]));
	  id++;
        }
        //particle[i+j*Npx] = new PARTICLE(i+j*Npx+1,color,r,v,material[material_id]);
        r += Rot * (sign*dx);
    }
    if (distribution == "hcp")
      r += Rot * (dy - sign*dx/2.);
    else 
      INTERNAL_ERROR;
    sign*=-1;
  }
  // crystal 2
  Out << "Particle.size = " << particle.size() << endl;
  int psize = particle.size();
  for (int ip = 0; ip < psize; ip++) {
    PARTICLE* p = particle[ip];
    if (p->x[1] > 0.5 && (p->x[1]-10.)*(p->x[1]-10.) + (p->x[0]-110.)*(p->x[0]-110.) <= 10.*10.) { 
      VECTOR r = p->x;
      r[1] = -r[1]+min-7;
      r[0] =  r[0]-.3;
      if (p->color == 1)  
	color = 3;
      else 
	color = 4;
      particle.add(new PARTICLE(id,color,r,v,material[0]));
      id++;
    }
  }
  Np = id-1;
 }
 }
 else // read_from_file
 {
  STRING ctl, str;
  ASCII_FILE* inp = new ASCII_FILE(input_file());
 
  inp->locate_at_level(3,"particles");
  if(!inp->ok) INPUT_ERROR("Cannot find ***particles in file");
   
  int dim = inp->getint(), 
      Np = inp->getint(),c,counter=0,id;
  VECTOR r(dim),v(dim),f(dim); 
  particle.resize(Np);
  double lmult = 1/material[0]->multiplier.length,
	 vmult = 1/material[0]->multiplier.velocity;  
  //lmult = 1, vmult = 1;
  Out << "Reading file: Np = " << Np <<  ", dim = " << dim << endl;
  for(;;) {
    if (counter < Np) {
      id = inp->getint();
      for (int i = 0; i < dim; i++)
        r[i] = inp->getnumeral();
      for (int i = 0; i < dim; i++)
        v[i] = inp->getnumeral();
      c = inp->getint();
      counter++;
      particle[counter-1] = new PARTICLE(id,c,r*lmult,v*vmult,material[0]);
//      particle[counter-1]->bc_x.resize(inp->getint());
//      particle[counter-1]->bc_v.resize(inp->getint());
//      particle[counter-1]->bc_f.resize(inp->getint());
    }
    else
      break;
  }
 }
}
void MD::define_rmin_rmax()
{
  rmin = rmax = particle[0]->x;
  for (int i = 1; i < particle.size(); i++)
  {
    for (int d = 0; d < dim; d++)
    {
      if ((particle[i]->x - rmax)[d] > 0.)
	rmax[d] = particle[i]->x[d];
      if ((-1*particle[i]->x + rmin)[d] > 0.)
	rmin[d] = particle[i]->x[d];
    }
  }
}
void MD::update_bc()
{

  VECTOR vi(2),vj(2);
  vi[0] = 1; vi[1] = 0;
  vj[0] = 0; vj[1] = 1;

/*
  for (int i = 0; i < applied_bc.size(); i++) {
    particle[applied_bc[i]]->bc_x += .04*dt*vj+0.01*dt*vi;
  }
*/

/*
  VECTOR ext_f(2,0.);
  if (itime < 1000)
    ext_f[1] = 1.*itime/1000.;
  else
    ext_f[1] = 1.;
*/

// Used for the impact
   /*
  if (itime == 1) {
    for (int i = 0; i < particle.size(); i++) {
      if (particle[i]->color == 3 || particle[i]->color == 4) {
	particle[i]->v[0] += 1.;
	particle[i]->v[1] += 1.5;
      }
    }
  }
*/
  

/*
  if (itime > 0.3*time_bc && itime < time_bc)
  {
     for (int i = 0; i < applied_bc.size(); i++)
        average_bc[i] += 1/0.7*inv_time_bc * particle[applied_bc[i]]->x;
     for (int i = 0; i < applied_bc0.size(); i++)
        average_bc0[i] += 1/0.7*inv_time_bc * particle[applied_bc0[i]]->x;
  }
  if (itime >= time_bc && itime % 10 == 0 && itime < 10000)
  { 
     for (int i = 0; i < applied_bc.size(); i++)
          particle[applied_bc[i]]->bc_x = particle[applied_bc[i]]->x + (average_bc[i]- particle[applied_bc[i]]->x)*(1.-exp(-(itime-time_bc)/200.));
     for (int i = 0; i < applied_bc0.size(); i++)
          particle[applied_bc0[i]]->bc_x = particle[applied_bc0[i]]->x + (average_bc0[i]-particle[applied_bc0[i]]->x)*(1.-exp(-(itime-time_bc)/200.));
  }
*/

  double force_y_left = 0, force_y_right = 0, du = 1.e-3, force_x_left = 0, force_x_right = 0, df = 0;

/*
  for (int i = 0; i < applied_bc.size(); i++) {
    particle[applied_bc[i]]->bc_x[0] += 0;
    particle[applied_bc[i]]->bc_x[1] += 0;
  }
  for (int i = 0; i < applied_bc0.size(); i++) {
    particle[applied_bc0[i]]->x[0] += .1;
//    particle[applied_bc0[i]]->x[1] += .1;
  }
*/
/*
  if (itime > 10000) // && itime % 10 == 0)
  {
    for (int i = 0; i < applied_bc0.size(); i++)
    {
      particle[applied_bc0[i]]->bc_x[0] += du;
      //particle[applied_bc0[i]]->bc_f = ext_f;
      force_x_right += particle[applied_bc0[i]]->F[0];
      force_y_right += particle[applied_bc0[i]]->F[1];
    }
    displacement += du;
  }
  file_displ_force << md_time << " " << displacement << " " << force_x_left << " " << force_x_right  << " " << force_y_left << " " << force_y_right << endl;
*/  
//  particle[0]->bc_x = particle[0]->x;
//  particle[20]->bc_x = particle[20]->x;
//  particle[1]->bc_x = particle[1]->x;
//  particle[12]->bc_x = particle[12]->x;

/*
  particle[9]->bc_x += .001*vi;
  particle[29]->bc_x -= .001*vj;
*/
//  particle[9]->color = 2;
//  particle[12]->color = 2;
 

  if (itime > 0 && itime % set_every == 0 && gamma_temperature != 0.) {
  double scale = (particle.size()*kB*get_temperature(md_time))/(kinetic_energy*pow(material[0]->multiplier.velocity,2)*(material[0]->multiplier.mass)), factor = sqrt(1+gamma_temperature*(scale-1));
  for (int i = 0; i < particle.size(); i++)
    particle[i]->v *= factor;
  }
 

//Heating-cooling
//  double scale=.99;
//  if (kinetic_energy > 1e-17) // && md_time < 2400.) 
//    scale = sqrt(1./kinetic_energy);
    //scale = particle.size()*sqrt(sqrt(md_time/50000.)/kinetic_energy);
//   if (kinetic_energy > 1e-7)
 
//       scale = (1.-exp(-pow(1.*(itime+10),0.3)));// sqrt((1000000.-itime)/1000000.);


/*
    if (itime % 10 == 0) {
      for (int i = 0; i < particle.size(); i++)
        particle[i]->v *= scale;
    }

  if (itime < 500) {
    scale = 0.95;
    if (itime % 10 == 0) {
      for (int i = 0; i < particle.size(); i++)
        particle[i]->v *= scale;
    }
  }
  else {
    scale = 0.99;
    if (itime % 50 == 0) {
      for (int i = 0; i < particle.size(); i++)
        particle[i]->v *= scale;
    }
  }
*/


/*
  if ( md_time > 200. && md_time < 400.)
  {
    for (int i = 0; i < particle.size(); i++)
      particle[i]->v *= 1.0001;
  }
  else if ( md_time > 600.)
  {
    for (int i = 0; i < particle.size(); i++)
      particle[i]->v /= 1.0001;
  }
*/
}
void MD::distribute_into_cells()
{
  if (!(rmin.size() == rmax.size() && rmax.size() == dim))
    ERROR("Before creating cells, please first define rmin and rmax");
// check Boundary conditions
      bool periodic_x = FALSE, periodic_y = FALSE, periodic_z = FALSE;
      for (int ibc = 0; ibc < bc.size(); ibc++) {
         if (bc[ibc]->get_type() == PERIODIC_X)
	   periodic_x = TRUE;
         if (bc[ibc]->get_type() == PERIODIC_Y)
	   periodic_y = TRUE;
         if (bc[ibc]->get_type() == PERIODIC_Z)
	   periodic_z = TRUE;
      }

// create cells 
  VECTOR vijk(dim,1.),vi(dim),vj(dim),vk(dim);
  vi[0] = 1; vi[1] = 0;
  vj[0] = 0; vj[1] = 1;
  if (dim > 2) {
    vi[2] = 0;
    vj[2] = 0;
    vk[0] = 0; vk[1] = 0; vk[2] = 1;
  }
  VECTOR area_size = (rmax-rmin), plus_vector(dim);
  plus_vector = 0;

  if (box.norm() == 0) {
	  if (periodic_x && !periodic_y)
	    plus_vector = material[0]->material_parameter[1]*pow(2.,1./6.)*vi + 3*cut_off*(vj);
	  else if (periodic_y && !periodic_x)
	    plus_vector = (sqrt(3)/2.*material[0]->material_parameter[1]*pow(2.,1./6.)*vj)+3*cut_off*(vi);
	  else if (periodic_x && periodic_y) {
	    plus_vector = (sqrt(3)/2.*material[0]->material_parameter[1]*pow(2.,1./6.)*vj + material[0]->material_parameter[1]*pow(2.,1./6.)*vi);
	  }
	  else 
	    plus_vector = 2*cut_off; //14*vijk*cut_off;

	   plus_vector = cut_off*(vi+0.1*vj);

	  periodic_x = TRUE;
	  periodic_y = TRUE;
	  plus_vector = (sqrt(3)/2.*dist_mult*material[0]->material_parameter[1]*pow(2.,1./6.)*vj + 0.5*dist_mult*material[0]->material_parameter[1]*pow(2.,1./6.)*vi);

	  area_size += plus_vector;
  }
  else 
    area_size = box;

  Nx = int(area_size[0] / (safe_coef_for_cells_x*cut_off)),
  Ny = int(area_size[1] / (safe_coef_for_cells_y*cut_off));
  if (dim > 2)
    Nz = int(area_size[2] / (safe_coef_for_cells_z*cut_off));
  else 
    Nz = 1;
  Sx = area_size[0]/double(Nx);
  Sy = area_size[1]/double(Ny);
 
  if (dim > 2)
    Sz = area_size[2]/double(Nz);
  else
    Sz = 0.;

  double cell_x,cell_y,cell_z; 
  VECTOR cell_size(dim),cell_corner(dim);
  cell.resize(Nx*Ny*Nz);
  CELL_POSITION pos;
  LIST<int> neighbours;
  LIST<VECTOR> vectors;
  for (int i = 0; i < Nx; i++) {
     for (int j = 0; j < Ny; j++) {
	for (int k = 0; k < Nz; k++) {
	  neighbours.resize(0);
          if (box.norm() == 0) {
	    if (dim > 2)
      	      cell_z = (rmin-0.5*plus_vector)[2]+k*Sz;
	    else
	      cell_z = 0.;
      	    cell_y = (rmin-0.5*plus_vector)[1]+j*Sy;
      	    cell_x = (rmin-0.5*plus_vector)[0]+i*Sx; 
          }
          else {
             cell_x = i*Sx;
             cell_y = j*Sy; 
             if (dim > 2)
	       cell_z = k*Sz;
	     else
	       cell_z = 0; 
	  }
	  if (dim == 2)
          {
      	    cell_size = Sx*vi + Sy*vj; 
      	    cell_corner = cell_x*vi + cell_y*vj;
          }
	  else 
          {
      	    cell_size = Sx*vi + Sy*vj + Sz*vk; 
	    cell_corner = cell_x*vi + cell_y*vj + cell_z*vk;
          }
	  if (dim == 2)
          {
	    neighbours.resize(0);
	    vectors.resize(0);
	    if (i>0 && i<Nx-1 && j>0 && j<Ny-1)
            {
		pos = MID_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i+1+(j+1)*Nx);
		vectors.add(vi+vj);

		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		neighbours.add(i-1+(j+1)*Nx);
		vectors.add(vj-vi);

		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		neighbours.add(i-1+(j-1)*Nx);
		vectors.add(-1*vi-vj);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		neighbours.add(i+1+(j-1)*Nx);
		vectors.add(vi-vj);
	    }
	    else if (i==0 && j>0 && j<Ny-1)
            {
		pos = LEFT_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i+1+(j+1)*Nx);
		vectors.add(vi+vj);

		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		neighbours.add(i+1+(j-1)*Nx);
		vectors.add(vi-vj);

		if (periodic_x)
		{
		  neighbours.add(i+Nx-1+(j+1)*Nx);
		  vectors.add(vj-vi);

		  neighbours.add(i+Nx-1+j*Nx);
		  vectors.add(-1*vi);

		  neighbours.add(i+Nx-1+(j-1)*Nx);
		  vectors.add(-1*vi-vj);
		}  
            }
	    else if (i==Nx-1 && j>0 && j<Ny-1)
            {
		pos = RIGHT_2D;
		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		neighbours.add(i-1+(j+1)*Nx);
		vectors.add(vj-vi);

		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		neighbours.add(i-1+(j-1)*Nx);
		vectors.add(-1*vi-vj);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		if (periodic_x)
		{
		  neighbours.add(i-Nx+1+(j+1)*Nx);
		  vectors.add(vi+vj);

		  neighbours.add(i-Nx+1+j*Nx);
		  vectors.add(vi);

		  neighbours.add(i-Nx+1+(j-1)*Nx);
		  vectors.add(vi-vj);
		}  
            }
	    else if (j==0 && i>0 && i<Nx-1)
	    {
		pos = BOT_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i+1+(j+1)*Nx);
	        vectors.add(vi+vj);

		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		neighbours.add(i-1+(j+1)*Nx);
		vectors.add(vj-vi);

		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		if (periodic_y)
		{
		  neighbours.add(i+1+Nx*(Ny-1));
		  vectors.add(vi-vj);

		  neighbours.add(i+Nx*(Ny-1));
		  vectors.add(-1*vj);

		  neighbours.add(i-1+Nx*(Ny-1));
		  vectors.add(-1*vi-vj);
		}
	    }
	    else if (j==Ny-1 && i>0 && i<Nx-1)
	    {
		pos = TOP_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		neighbours.add(i-1+(j-1)*Nx);
		vectors.add(-1*vi-vj);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		neighbours.add(i+1+(j-1)*Nx);
		vectors.add(vi-vj);

		if (periodic_y)
		{
		  neighbours.add(i+1);
  	          vectors.add(vi+vj);

		  neighbours.add(i);
		  vectors.add(vj);

		  neighbours.add(i-1);
		  vectors.add(vj-vi);
		}
	    }
	    else if (i==0 && j==0)
	    {
		pos = BOT_LEFT_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i+1+(j+1)*Nx);
	        vectors.add(vi+vj);

		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		if (periodic_x || periodic_y)
		{
		  neighbours.add(cell.size()-1);
		  vectors.add(-1*vi-vj);
                }
                if (periodic_x)
                {		
		  neighbours.add(Nx-1);
		  vectors.add(-1*vi);

		  neighbours.add(2*Nx-1);
		  vectors.add(vj-vi);
		}
		if (periodic_y)
		{
		  neighbours.add(Nx*(Ny-1));
		  vectors.add(-1*vj);

		  neighbours.add(Nx*(Ny-1)+1);
		  vectors.add(vi-vj);
		}
	    }
	    else if (i==0 && j==Ny-1)
	    {
		pos = TOP_LEFT_2D;
		neighbours.add(i+1+j*Nx);
		vectors.add(vi);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		neighbours.add(i+1+(j-1)*Nx);
		vectors.add(vi-vj);

		if (periodic_x || periodic_y)
		{
		  neighbours.add(Nx-1);
		  vectors.add(vj-vi);
                }
                if (periodic_x)
                {		
		  neighbours.add(Nx*Ny-1);
		  vectors.add(-1*vi);

		  neighbours.add(Nx*(Ny-1)-1);
		  //WAS neighbours.add(Nx*(Ny-1));
		  vectors.add(-1*vi-vj);
		}
		if (periodic_y)
		{
		  neighbours.add(i);
		  vectors.add(vj);

		  neighbours.add(i+1);
  	          vectors.add(vi+vj);
		}
	    }
	    else if (i==Nx-1 && j==0)
	    {
		pos = BOT_RIGHT_2D;	
		neighbours.add(i+(j+1)*Nx);
		vectors.add(vj);

		neighbours.add(i-1+(j+1)*Nx);
		vectors.add(vj-vi);

		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		if (periodic_x || periodic_y)
		{
		  neighbours.add(Nx*(Ny-1));
		  //WAS neighbours.add(Nx*(Ny-1)+1);
		  vectors.add(vi-vj);
                }
                if (periodic_x)
                {		
		  neighbours.add(0);
		  vectors.add(vi);

		  neighbours.add(Nx);
	          vectors.add(vi+vj);
		}
		if (periodic_y)
		{
		  neighbours.add(Nx*Ny-1);
		  vectors.add(-1*vj);

		  neighbours.add(Nx*Ny-2);
		  vectors.add(-1*vi-vj);
		}
            }
	    else if (i==Nx-1 && j==Ny-1)
	    {
		pos = TOP_RIGHT_2D;
		neighbours.add(i-1+j*Nx);
		vectors.add(-1*vi);

		neighbours.add(i-1+(j-1)*Nx);
		vectors.add(-1*vi-vj);

		neighbours.add(i+(j-1)*Nx);
		vectors.add(-1*vj);

		if (periodic_x || periodic_y)
		{
		  neighbours.add(0);
	          vectors.add(vi+vj);
                }
                if (periodic_x)
                {		
		  neighbours.add(Nx*(Ny-1));
		  vectors.add(vi);

		  neighbours.add(Nx*(Ny-2));
		  vectors.add(vi-vj);
		}
		if (periodic_y)
		{
		  neighbours.add(Nx-1);
		  vectors.add(vj);

		  neighbours.add(Nx-2);
		  vectors.add(vj-vi);
		}
	    }
	    else
		ERROR("Should not be here");
          }
	  else 
	    ERROR("Not implemented neighbouring cells and vectors yet");
      	  cell[i+Nx*j+Nx*Ny*k] = new CELL(cell_corner,cell_size,Nx,Ny,Nz,pos,neighbours,vectors);
        }
     }
  }

// Distribute particles in cells
  int cell_number;
  if (dim == 2)
  {
/*
    VECTOR ds = material[0]->material_parameter[1]*pow(2.,1./6.); 
    if (periodic_x && !periodic_y)
    {
      for (int i = 0; i < particle.size(); i++)
      {
        cell_number = int((particle[i]->x + vi*ds + cut_off*vj - rmin)[0]/Sx)+Nx*int((particle[i]->x + vi*ds + vj*cut_off - rmin)[1]/Sy);
        cell[cell_number]->particles.add(particle[i]);
      }
    }
    if (!periodic_x && periodic_y)
    {
      for (int i = 0; i < particle.size(); i++)
      {
        cell_number = int((particle[i]->x + vj*sqrt(3)/2.*ds + cut_off*vi - rmin)[0]/Sx)+Nx*int((particle[i]->x + sqrt(3)/2.*vj*ds + vi*cut_off - rmin)[1]/Sy);
        cell[cell_number]->particles.add(particle[i]);
      }
    }
    if (periodic_x && periodic_y)
    {
      for (int i = 0; i < particle.size(); i++)
      {
        cell_number = int((particle[i]->x +  vi*ds + vj*sqrt(3)/2.*ds - rmin)[0]/Sx)+Nx*int((particle[i]->x + vi*ds + vj*sqrt(3)/2.*ds - rmin)[1]/Sy);
        cell[cell_number]->particles.add(particle[i]);
      }
    }
    else
    {
      for (int i = 0; i < particle.size(); i++)
      {
        cell_number = int((particle[i]->x + vijk*cut_off - rmin)[0]/Sx)+Nx*int((particle[i]->x + vijk*cut_off - rmin)[1]/Sy);
        cell[cell_number]->particles.add(particle[i]);
      }
    }
*/
    for (int i = 0; i < particle.size(); i++)
    {
      cell_number = int((particle[i]->x + 0.5*plus_vector - rmin)[0]/Sx)+Nx*int((particle[i]->x + 0.5*plus_vector - rmin)[1]/Sy);
      cell[cell_number]->particles.add(particle[i]);
    }

/*      if ((cell_number+1)%Nx != 0)
      {
	cell[cell_number].add_neighbour(cell_number+1);
	if (cell_number >= Nx)	
	  cell[cell_number].add_neighbour(cell_number-Nx+1);
	if (cell_number < Nx*(Ny-1))
	  cell[cell_number].add_neighbour(cell_number+Nx+1);
      }
      else if (periodic_x)
      {
	cell[cell_number].add_neighbour(cell_number-Nx+1);
	if (cell_number >= Nx)	
	  cell[cell_number].add_neighbour(cell_number-2*Nx+1);
	else 
	  cell[cell_number].add_neighbour(cell.size()-Nx);	
	if (cell_number < Nx*(Ny-1))
	  cell[cell_number].add_neighbour(cell_number+1);
	else
          cell[cell_number].add_neighbour(cell_number+1);
      }
      if ((cell_number)%Nx != 0)
      {
	cell[cell_number].add_neighbour(cell_number-1);	
	if (cell_number >= Nx)	
	  cell[cell_number].add_neighbour(cell_number-Nx-1);
	if (cell_number < Nx*(Ny-1))
	  cell[cell_number].add_neighbour(cell_number+Nx-1);		
      }
      if (cell_number >= Nx)
      {
	cell[cell_number].add_neighbour(cell_number-Nx);		
      }
      if (cell_number < Nx*(Ny-1))
      {
	cell[cell_number].add_neighbour(cell_number+Nx);		
      }
    }
*/
  }
  else if (dim == 3) 
  {
    for (int i = 0; i < particle.size(); i++)
    {
        cell_number = int((particle[i]->x + vijk*cut_off - rmin)[0]/Sx)
             +Nx*int((particle[i]->x + vijk*cut_off - rmin)[1]/Sy)
      +Nx*Ny*int((particle[i]->x + vijk*cut_off - rmin)[2]/Sz);
      cell[cell_number]->particles.add(particle[i]); // does not work because each CELL has to be constructed
    }
  }

}
/*
void MD::print(Zofstream& file, STRING str)
{
  if (str=="particle_coord")
  {
//    for (int i = 0; i < particle.size(); i++)
//        file << particle[i]->x[0] << " " << particle[i]->x[1] << endl;
     if (dim == 2)
     {
       for (int ic = 0; ic < cell.size(); ic++)
         for (int ip = 0; ip < cell[ic]->particles.size(); ip++)
           file << cell[ic]->particles[ip]->x[0] << " " << cell[ic]->particles[ip]->x[1] << endl;
     }
     else 
     {
       for (int ic = 0; ic < cell.size(); ic++)
         for (int ip = 0; ip < cell[ic]->particles.size(); ip++)
           file << cell[ic]->particles[ip]->x[0] << " " << cell[ic]->particles[ip]->x[1] << " " << cell[ic]->particles[ip]->x[2] << endl;
     }

  }
  else
    ERROR("Unknown print option");
}
void MD::print_by_color(ARRAY<Zofstream*>& file, STRING str)
{
  if (str=="particle_coord")
  {
     if (dim == 2)
     {
       for (int ic = 0; ic < cell.size(); ic++)
       {
         for (int ip = 0; ip < cell[ic]->particles.size(); ip++)
	 {
           *(file[cell[ic]->particles[ip]->color-1]) << cell[ic]->particles[ip]->x[0] << " " << cell[ic]->particles[ip]->x[1] << endl;
	 }
       }
     }
     else 
     {
       for (int ic = 0; ic < cell.size(); ic++)
         for (int ip = 0; ip < cell[ic]->particles.size(); ip++)
           *(file[cell[ic]->particles[ip]->color-1]) << cell[ic]->particles[ip]->x[0] << " " << cell[ic]->particles[ip]->x[1] << " " << cell[ic]->particles[ip]->x[2] << endl;
     }

  }
  else
    ERROR("Unknown print option");
}

*/
void MD::MaxwellBoltzmanDistribution(double T)
{
  srand(seed); 
  double mult, constt = sqrt(kB*T/pow(material[0]->multiplier.velocity,2.));
  Out << "constt = " << constt << endl;
  //double mult, constt = sqrt(kB*T);
  for (int i = 0; i < particle.size(); i++) {
    mult = sqrt(1/(particle[i]->mat->mass*material[0]->multiplier.mass)); 
    particle[i]->v = constt*mult*GaussDeviate();
  }
}
VECTOR MD::GaussDeviate() {
  VECTOR result(dim), a(dim);
  double s,r;

  do {
    for (int j = 0; j < dim; j++) 
      a[j] = 2*(rand() / double (RAND_MAX)) - 1.; 
    r = a[0]*a[0] + a[1]*a[1];
  } while (r >= 1.);
  s = sqrt(-2.*log(r)/r);
  for (int j = 0; j < dim; j++)
    result[j] = s*a[j];

// Original Box-Muller method
/*
  a[0] = rand() / double (RAND_MAX);
  a[1] = rand() / double (RAND_MAX);
  result[0] = sqrt(-2*log(a[0]))*cos(2*M_PI*a[1]);
  result[1] = sqrt(-2*log(a[0]))*sin(2*M_PI*a[1]);
*/
  return result;
}

